from typing import Type

from citrine._serialization import properties
from citrine._serialization.polymorphic_serializable import PolymorphicSerializable
from citrine.informatics.predictors import Predictor


class PredictorNode(PolymorphicSerializable["PredictorNode"], Predictor):
    """An individual compute node within a Predictor.

    A PredictorNode cannot be registered to the Citrine Platform by itself
    and must be included as a component within a GraphPredictor to be used.

    """

    name = properties.String("name")
    description = properties.Optional(properties.String(), "description")

    @classmethod
    def get_type(cls, data) -> Type['PredictorNode']:
        """Return the subtype."""
        from .expression_predictor import ExpressionPredictor
        from .molecular_structure_featurizer import MolecularStructureFeaturizer
        from .ingredients_to_formulation_predictor import IngredientsToFormulationPredictor
        from .label_fractions_predictor import LabelFractionsPredictor
        from .simple_mixture_predictor import SimpleMixturePredictor
        from .ingredient_fractions_predictor import IngredientFractionsPredictor
        from .auto_ml_predictor import AutoMLPredictor
        from .mean_property_predictor import MeanPropertyPredictor
        from .chemical_formula_featurizer import ChemicalFormulaFeaturizer
        type_dict = {
            "AnalyticExpression": ExpressionPredictor,
            "MoleculeFeaturizer": MolecularStructureFeaturizer,
            "IngredientsToSimpleMixture": IngredientsToFormulationPredictor,
            "MeanProperty": MeanPropertyPredictor,
            "LabelFractions": LabelFractionsPredictor,
            "SimpleMixture": SimpleMixturePredictor,
            "IngredientFractions": IngredientFractionsPredictor,
            "ChemicalFormulaFeaturizer": ChemicalFormulaFeaturizer,
            "AutoML": AutoMLPredictor,
        }
        typ = type_dict.get(data['type'])
        if typ is not None:
            return typ
        else:
            raise ValueError(
                '{} is not a valid predictor node type. '
                'Must be in {}.'.format(data['type'], type_dict.keys())
            )



from functools import partial
from typing import Iterator

from citrine._rest.collection import Collection, ResourceType


class AdminCollection(Collection[ResourceType]):
    """Abstract class for representing collections of REST resources with as_admin access."""

    def list(
        self, *, per_page: int = 100, as_admin: bool = False
    ) -> Iterator[ResourceType]:
        """
        Paginate over the elements of the collection.

        Leaving page and per_page as default values will yield all elements in the
        collection, paginating over all available pages.

        Parameters
        ---------
        per_page: int, optional
            Max number of results to return per page. Default is 100.  This parameter
            is used when making requests to the backend service.  If the page parameter
            is specified it limits the maximum number of elements in the response.

        as_admin: bool, optional
            Whether this request should be made as an admin (returning all teams,
            rather than only those to which the user belongs).

        Returns
        -------
        Iterator[ResourceType]
            An iterator that can be used to loop over all the resources in this collection.
            Use list() to force evaluation of all results into an in-memory list.

        """
        fetcher = partial(
            self._fetch_page, additional_params={"as_admin": "true"} if as_admin else {}
        )
        return self._paginator.paginate(
            page_fetcher=fetcher,
            collection_builder=self._build_collection_elements,
            per_page=per_page,
        )



from typing import Optional

from citrine._serialization import properties
from citrine._serialization.serializable import Serializable
from citrine.informatics.catalyst.language_model import LanguageModelChoice


class InsightsRequest(Serializable["InsightsRequest"]):
    """A query to the model insights."""

    question = properties.String("question")
    temperature = properties.Optional(properties.Float, "temperature", default=0.0)
    language_model = properties.Optional(
        properties.Enumeration(LanguageModelChoice),
        "language_model",
        default=LanguageModelChoice.GPT_35_TURBO,
    )
    n_documents = properties.Optional(properties.Integer, "n_documents", default=5)
    response_size = properties.Optional(properties.Integer, "response_size", default=100)

    def __init__(
        self,
        *,
        question: str,
        temperature: Optional[float] = 0.0,
        language_model: Optional[LanguageModelChoice] = LanguageModelChoice.GPT_35_TURBO,
        n_documents: Optional[int] = 5,
        response_size: Optional[int] = 100,
    ):
        self.question = question
        self.temperature = temperature
        self.language_model = language_model
        self.n_documents = n_documents
        self.response_size = response_size


class InsightsResponse(Serializable["InsightsResponse"]):
    """A response from the model insights."""

    response = properties.String("response")
    relevant_documents = properties.Boolean("relevant_documents")
    document_metadata = properties.List(
        properties.Mapping(properties.String, properties.String), "document_metadata"
    )



from citrine._serialization import properties
from citrine._serialization.serializable import Serializable
from citrine.informatics.constraints.constraint import Constraint
from citrine.informatics.descriptors import FormulationDescriptor

__all__ = ['IngredientFractionConstraint']


class IngredientFractionConstraint(Serializable['IngredientFractionConstraint'], Constraint):
    """Represents a constraint on an ingredient fraction in a formulation.

    Parameters
    ----------
    formulation_descriptor: FormulationDescriptor
        descriptor to constrain
    ingredient: str
        name of the ingredient to constrain
    min: float
        minimum ingredient value
    max: float
        maximum ingredient value
    is_required: bool, optional
        whether this ingredient is required.
        If ``True``, the ingredient must be present and its value must be within the
        specified range. if ``False``, the ingredient must be within the specified range only if
        it's present in the formulation, i.e., the value can be 0 or on the range ``[min, max]``.

    """

    formulation_descriptor = properties.Object(FormulationDescriptor, 'formulation_descriptor')
    ingredient = properties.String('ingredient')
    min = properties.Optional(properties.Float, 'min')
    max = properties.Optional(properties.Float, 'max')
    is_required = properties.Boolean('is_required')
    typ = properties.String('type', default='IngredientFractionConstraint')

    def __init__(self, *,
                 formulation_descriptor: FormulationDescriptor,
                 ingredient: str,
                 min: float,
                 max: float,
                 is_required: bool = True):
        self.formulation_descriptor: FormulationDescriptor = formulation_descriptor
        self.ingredient: str = ingredient
        self.min: float = min
        self.max: float = max
        self.is_required: bool = is_required

    def __str__(self):
        return (
            f"<IngredientFractionConstraint "
            f"'{self.formulation_descriptor.key}'::'{self.ingredient}'>"
        )



from setuptools import setup, find_packages
from os import path

# read the contents of your README file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

about = {}
with open(path.join(this_directory, 'src', 'citrine', '__version__.py'), 'r') as f:
    exec(f.read(), about)

setup(name='citrine',
      # Update this in src/citrine/__version__.py
      version=about['__version__'],
      python_requires='>=3.8',
      url='http://github.com/CitrineInformatics/citrine-python',
      description='Python library for the Citrine Platform',
      long_description=long_description,
      long_description_content_type='text/markdown',
      author='Citrine Informatics',
      package_dir={'': 'src'},
      packages=find_packages(where='src'),
      install_requires=[
          "requests>=2.31.0,<3",
          "pyjwt>=2,<3",
          "arrow>=1.0.0,<2",
          "gemd>=2.0.0,<3",
          "boto3>=1.34.35,<2",
          "deprecation>=2.1.0,<3",
          "urllib3>=1.26.18,<3",
          "tqdm>=4.27.0,<5",
          "pint<=0.20; python_version < '3.9'",
          "pint<0.24; python_version >= '3.9'"
      ],
      extras_require={
          "../tests": [
              "factory-boy>=3.3.0,<4",
              "mock>=5.1.0,<6",
              "pandas>=2.0.3,<3",
              "pytest>=8.0.0,<9",
              "pytz>=2024.1",
              "requests-mock>=1.11.0,<2",
          ]
      },
      classifiers=[
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.8',
          'Programming Language :: Python :: 3.9',
          'Programming Language :: Python :: 3.10',
          'Programming Language :: Python :: 3.11',
      ],
      )



from gemd.enumeration.base_enumeration import BaseEnumeration


class LanguageModelChoice(BaseEnumeration):
    """The options for which model to use to answer a query."""

    GPT_35_TURBO = "gpt-3.5-turbo"
    GPT_35_TURBO_16K = "gpt-3.5-turbo-16k"
    GPT_4 = "gpt-4"
    GPT_4_32K = "gpt-4-32k"
